import requests

class STATUS_PEDIDO(object):
    AGUARDANDO_PAGAMENTO = 'pedido_esperando_pagamento'
    FATURADO = 'pedido_faturado'
    ENVIADO = 'pedido_enviado'
    ENTREGUE = 'pedido_entregue'
    CANCELADO = 'pedido_cancelado'


class Axado(object):
    def __init__(self, api_key, base_url='https://api.axado.com.br/v2/'):
        self._key = api_key
        self._base_url = base_url
        self._headers = {
            'Authorization': self._key,
        }
        self._params = {'token': self._key}

    @staticmethod
    def _json_wrapper(response):
        try:
            return response.json()
        except Exception as e:
            return {"error": u"{}".format(e)}

    @staticmethod
    def _digits_only(value):
        return "".join([_ for _ in value if _.isdigit()])

    def _requests_wrapper(self, method, *args, **kwargs):
        headers = kwargs.get('headers', {})
        headers.update(self._headers)
        kwargs.update({'headers': headers})
        try:
            r = getattr(requests, method)(*args, **kwargs)
        except Exception as e:
            return {"error": u"{}".format(e)}
        if 200 <= r.status_code <= 300:
            return self._json_wrapper(r)
        return {
            "error": "API error",
            "status_code": r.status_code,
            "data": self._json_wrapper(r)
        }

    def get(self, *args, **kwargs):
        return self._requests_wrapper('get', *args, **kwargs)

    def post(self, *args, **kwargs):
        return self._requests_wrapper('post', *args, **kwargs)

    def patch(self, *args, **kwargs):
        return self._requests_wrapper('patch', *args, **kwargs)

    def cotacao(self, cep_origem, cep_destino, volumes, valor_notafiscal, prazo_adicional=0,
                preco_adicional=0, fretegratis=False):
        if '-' in cep_origem:
            cep_origem = self._digits_only(cep_origem)
        if '-' in cep_destino:
            cep_destino = self._digits_only(cep_destino)

        return self.post(
            self._base_url + 'consulta',
            params={'token': self._key},
            json={
                "cep_origem": cep_origem,
                "cep_destino": cep_destino,
                "valor_notafiscal": valor_notafiscal,
                "prazo_adicional": prazo_adicional,
                "preco_adicional": preco_adicional,
                "volumes": volumes,
                "fretegratis": fretegratis,
            }
        )

    def cotacao_consultar(self, token):
        return self.get(self._base_url + 'consulta/' + token, params=self._params)

    def pedido(self, consulta_token, cotacao_codigo, numero, status):
        params={'token': self._key}
        return self.post(
            self._base_url + 'pedido/',
            params=self._params,
            json={
                "consulta_token": consulta_token,
                "cotacao_codigo": cotacao_codigo,
                "numero": numero,
                "status": status,
            },
        )

    def pedido_consultar(self, pedido):
        return self.get(self._base_url + 'pedido/' + pedido, params=self._params)

    def pedido_atualizar(self, pedido, status):
        return self.patch(
            self._base_url + 'pedido/' + pedido,
            params=self._params,
            json={
                "status": status,
            }
        )

    def pedido_adicionar_nf(self, pedido, chave_acesso, cnpj_emitente, data_emissao, numero, serie,
                            valor_icms, valor_total, valor_frete):
        return self.patch(
            self._base_url + 'pedido/{}/notafiscal'.format(pedido),
            params=self._params,
            json={
                "chave_acesso": chave_acesso,
                "cnpj_emitente": cnpj_emitente,
                "data_emissao": data_emissao,
                "numero": numero,
                "serie": serie,
                "valor_icms": valor_icms,
                "valor_total": valor_total,
                "valor_frete": valor_frete,
            },
        )



import random
import unittest
from axado.api import Axado, STATUS_PEDIDO
try:
    from settings import KEY
except ImportError:
    print('''Please, create a settings.py file containing "KEY='XXXXXX'"
    where 'XXXXXX' is your API key obtained at http://www.axado.com.br''')


class TestConsulta(unittest.TestCase):
    def setUp(self):
        self.api = Axado(KEY)

    def test_digits(self):
        self.assertNotIn('-', self.api._digits_only('12345-678'))

    def test_consulta(self):
        result = self.api.cotacao(
            '04544051',
            '04568000',
            [
                {
                    "sku": "A-01",
                    "quantidade": "1",
                    "preco": "15,20",
                    "altura": "35",
                    "comprimento": "10",
                    "largura": "30",
                    "peso": "2",
                }
            ],
            "15,20",
        )
        self.assertIn('consulta_token', result)
        return result

    def test_consulta_cep_with_dash(self):
        result = self.api.cotacao(
            '04544-051',
            '04568-000',
            [
                {
                    "sku": "A-01",
                    "quantidade": "1",
                    "preco": "15,20",
                    "altura": "35",
                    "comprimento": "10",
                    "largura": "30",
                    "peso": "2",
                }
            ],
            "15,20",
        )
        self.assertIn('consulta_token', result)

    def test_error_wrapping(self):
        self.assertIn('error', self.api.cotacao('', '', [], ''))

    def test_consultar_cotacao(self):
        result = self.test_consulta()
        token = result['consulta_token']
        self.assertIn('consulta_token', self.api.cotacao_consultar(token))

    def test_pedido(self):
        result = self.test_consulta()
        token = result['consulta_token']
        result = self.api.pedido(
            consulta_token=token,
            cotacao_codigo="1",
            numero=str(random.randint(1, 100000)),
            status=STATUS_PEDIDO.AGUARDANDO_PAGAMENTO,
        )
        self.assertIn('numero', result)
        self.assertIn('consultas', result)
        self.assertIn('status', result)
        self.assertIn('data_cadastro', result)
        return result

    def test_consultar_pedido(self):
        result = self.test_pedido()
        pedido = result['numero']
        result = self.api.pedido_consultar(pedido)

    def test_atualizar_pedido(self):
        result = self.test_pedido()
        pedido = result['numero']
        result = self.api.pedido_atualizar(pedido, STATUS_PEDIDO.CANCELADO)

    def test_adicionar_nota_fiscal_ao_pedido(self):
        result = self.test_pedido()
        pedido = result['numero']
        result = self.api.pedido_adicionar_nf(
            pedido=pedido,
            chave_acesso='01234567890123456789012345678901234567891234',
            cnpj_emitente='12345678901234',
            data_emissao='2015-01-13T23:01:22Z',
            numero='12345',
            serie='3',
            valor_icms='0',
            valor_total='71.89',
            valor_frete='1.99',
        )
        self.assertIn('chave_acesso', result)
        self.assertIn('numero', result)
        self.assertIn('serie', result)
        return result


if __name__ == '__main__':
    unittest.main()


from setuptools import setup

setup(
  name = 'axado',
  packages = ['axado'],
  version = '0.1',
  description = 'Python bindings to the Axado API (https://developers.axado.com.br/)',
  author = 'Paulo Scardine',
  author_email = 'paulos@xtend.com.br',
  url = 'https://github.com/scardine/python-axado',
  download_url = 'https://github.com/scardine/python-axado/archive/0.1.tar.gz',
  keywords = ['axado', 'api'],
  classifiers = [],
  install_requires=[
    'requests',
  ],
)